package org.olga.rebus.calculations;

import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.olga.rebus.calculations.Result.WordResult;
import org.olga.rebus.structure.*;
import org.olga.rebus.structure.Rebus.VisibleWord;

/**
 *  -     . 
 *  . 
 * 
 * @author Olga
 * @see Rebus
 *
 */
public class RebusEnumerator {
	//  
	private ConcurrentLinkedQueue<AbstractWord> myUncalculatedWords 
						= new ConcurrentLinkedQueue<AbstractWord>();
	//-  
	private HashMap<AbstractWord, Integer> myCalculatedWords 
						= new HashMap<AbstractWord, Integer>();
	//,    
	private Context myContext = new Context();
	
	//,  
	private IRebus myRebus;
	
	//,      
	//  
	private ValuesGetter myValuesGetter;
		
	//      
	private StringBuffer myResults = new StringBuffer();
	
	//,     
	private Result myResult;
	
	
	/**
	 *  .
	 * 
	 * @param rebus ,   
	 * @param words ,     
	 * 				   
	 */
	public RebusEnumerator(IRebus rebus/*, AbstractWord[] words*/) {
		myRebus = rebus;
		//myWords = words;
		myResult = new Result(rebus.getVisibleWords());
	}
	
	/*
	 *    
	 */
	private void createMyCalculatedWords() {
		//    HashMap
		for(AbstractRelation relation: myRebus.getRelations()) {
			AbstractWord op1 = relation.getOp1();
			AbstractWord op2 = relation.getOp2();
			AbstractWord result = relation.getResult();
			
			int op1Val = op1.getValue(myContext);
			int op2Val = op2.getValue(myContext);
			
			if (op1Val != -1) {
				myCalculatedWords.put(op1, op1Val);
			} else {
				myUncalculatedWords.add(op1);
			}
			if (op2Val != -1) {
				myCalculatedWords.put(op2, op2Val);
			} else {
				myUncalculatedWords.add(op2);
			}
			if (result != null) {
				int resultVal = result.getValue(myContext);
				if (resultVal != -1) {
					myCalculatedWords.put(result, resultVal);
				} else {
					myUncalculatedWords.add(result);
				}
			}
		}
	}
	
	/*
	 *          .
	 *  false,     .
	 */
	private boolean addWordOnValue(AbstractWord word, int value) {
		if (value < 0) {
			return false;
		}
		String str = String.valueOf(value);
		if (word.getLength() != str.length()) {
			return false;
		}
		for (int i=0; i < str.length(); i++) {
			AbstractSymbol s = word.getSymbol(i);
			int val;
			if (s instanceof NumberSymbol) {
				val = ((NumberSymbol) s).getValue();
			} else {
				val = myContext.getSymbolValue(s);
			}
			int num = Integer.parseInt(String.valueOf(str.charAt(i)));
			if (val == -1) {
				if (!myContext.addSymbol(s, num)) {
					return false;
				}
			} else {
				if (val != num) {
					return false;
				}
			}
		}
		return true;
	}
	
	/*
	 *         ,
	 *   .     -
	 *  . 
	 *  false,     -
	 * . 
	 */
	private boolean calculateWords() {
		for (AbstractRelation relation: myRebus.getRelations()) {
			AbstractWord op1 = relation.getOp1();
			AbstractWord op2 = relation.getOp2();
			AbstractWord result = relation.getResult();
			boolean isOp1 = myCalculatedWords.containsKey(op1);
			boolean isOp2 = myCalculatedWords.containsKey(op2);
			if (result == null) {
				if (isOp1 && !isOp2) {
					int o1 = myCalculatedWords.get(op1);
					this.addWordOnValue(op2, o1);
					myCalculatedWords.put(op2, o1);
				} else if (!isOp1 && isOp2) {
					int o2 = myCalculatedWords.get(op2);
					this.addWordOnValue(op1, o2);
					myCalculatedWords.put(op1, o2);
				}
				continue;
			}
			boolean isResult = myCalculatedWords.containsKey(result);
			if (isOp1 && isOp2 && isResult) {
				if (!(relation.getThird(1, 2, myCalculatedWords.get(op1), myCalculatedWords.get(op2)) == myCalculatedWords.get(result))) {
					return false;
				}
			} else if (isOp1 && isOp2) {
				int res = relation.getThird(1, 2, myCalculatedWords.get(op1), myCalculatedWords.get(op2));
				if (result instanceof DummyWord) {
					myCalculatedWords.put(result, res);
					continue;
				}
				if (!this.addWordOnValue(result, res)) {
					return false;
				}
			} else if (isOp1 && isResult) {
				int o2 = relation.getThird(1, 3, myCalculatedWords.get(op1), myCalculatedWords.get(result));				
				if (op2 instanceof DummyWord) {
					myCalculatedWords.put(op2, o2);
					continue;
				}
				if (!this.addWordOnValue(op2, o2)) {
					return false;
				}	
			} else if (isOp2 && isResult) {
				int o1 = relation.getThird(2, 3, myCalculatedWords.get(op2), myCalculatedWords.get(result));				
				if (op1 instanceof DummyWord) {
					myCalculatedWords.put(op1, o1);
					continue;
				}
				if(!this.addWordOnValue(op1, o1)) {
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * ,     .
	 *     myResults
	 */
	public 	boolean calculate() {
		this.clear();
		this.createMyContext();
		if (!this.nextMyContext()) {
			return false;
		}
		int queueLength1 = 0;
		boolean isFirst = true;
		do {
			while (!myUncalculatedWords.isEmpty() || isFirst) {
				queueLength1 = myUncalculatedWords.size();
				if (this.calculateWords()) {
					isFirst = false;
					int k = myUncalculatedWords.size();
					while (k-- > 0) {
						AbstractWord word = myUncalculatedWords.poll();
						if (myCalculatedWords.containsKey(word)) {
							continue;
						}
						int value = word.getValue(myContext);
						if (value != -1) {
							myCalculatedWords.put(word, value);
						} else {
							myUncalculatedWords.add(word);
						}
					}
					if ((queueLength1 == myUncalculatedWords.size()) && (queueLength1 != 0)) {
						return false;
					}
				} else {
					if (!nextMyContext()) {
						return true;
					}
					isFirst = true;
				}
			}	
			isFirst = true;
			boolean f = true;
			Collection<VisibleWord> words = myRebus.getVisibleWords();
			for (VisibleWord word: words) {
				if (myContext.getSymbolValue(word.getElement().getSymbol(0)) == 0) {
					f = false;
					break;
				}
			}
			if (f) {
				for (AbstractRelation relation: myRebus.getRelations()) {
					AbstractWord op1 = relation.getOp1();
					AbstractWord op2 = relation.getOp2();
					AbstractWord result = relation.getResult();
					myResults.append(op1);
					myResults.append(" ");
					myResults.append(relation.getSign());
					myResults.append(" ");
					myResults.append(op2);
					if (result != null) {
						myResults.append(" = ");
						myResults.append(result);
					}
					myResults.append("\n");
					myResults.append(myCalculatedWords.get(op1));
					myResults.append(" ");
					myResults.append(relation.getSign());
					myResults.append(" ");
					myResults.append(myCalculatedWords.get(op2));
					
					if (result != null) {
						myResults.append(" = ");
						myResults.append(myCalculatedWords.get(result));
						myResults.append("\n");
					}
					myResults.append("\n");
				}
				myResult.add(myContext);
			}
		} while(nextMyContext());
		return true;
	}
	
	/**
	 * ,    .
	 * 
	 * @param r  .
	 */
	public void setRebus(IRebus r) {
		myRebus = r;
		this.clear();
		myResult = new Result(r.getVisibleWords());
	}
	
	/*
	 * ,    ,
	 *     .
	 */
	private void clear() {
		myUncalculatedWords.clear();
		myCalculatedWords.clear();
		myContext.clear();
	}
	
	/*
	 * ,  .     
	 *  (..    ).
	 */
	private boolean nextMyContext() {
		int l = 0;
		int k = 0;
		this.clear();
		if (myValuesGetter.next()) {
			Collection<AbstractWord> words = myRebus.getEnumWords();
			for (AbstractWord word: words) {
				for (int j=0; j < word.getLength(); j++) {
					AbstractSymbol symbol = word.getSymbol(j);
					if (!(symbol instanceof NumberSymbol)) {
						if (!myContext.containsSymbol(symbol)) {
							if (symbol instanceof LetterSymbol) {
								myContext.addSymbol(symbol, myValuesGetter.getValue1(l++));
							} else {
								myContext.addSymbol(symbol, myValuesGetter.getValue2(k++));
							}
							
						}
					}
				}
			}
			this.createMyCalculatedWords();
			return true;
		} else {
			return false;
		}
	}
	
	/*
	 * ,  .   ,
	 *     -   .
	 */
	private void createMyContext() {
		int l = 0;
		int k = 0;
		
		Collection<AbstractWord> words = myRebus.getEnumWords();
		for (AbstractWord word: words) {
			for (int j=0; j < word.getLength(); j++) {
				AbstractSymbol symbol = word.getSymbol(j);
				if (!(symbol instanceof NumberSymbol)) {
					if (!myContext.containsSymbol(symbol)) {
						myContext.addSymbol(symbol, l + k);
						if (symbol instanceof LetterSymbol) {
							l++;
						} else {
							k++;
						}
					}
				}
			}
		}
		myValuesGetter = new ValuesGetter(l, k);
		this.createMyCalculatedWords();
	}
	
	/**
	 * ,     .
	 * 
	 * @return    
	 */
	public String getResults() {
		return myResults.toString();
	}
	
	/**
	 * ,       
	 * .
	 * 
	 * @return   .
	 */
	public Iterator<LinkedList<WordResult>> getResult() {
		return myResult.getResults();
	}
	
	/**
	 * ,    .
	 * 
	 * @return  .
	 */
	public int getNumberResults() {
		return myResult.getSize();
	}
}
