package org.olga.rebus.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.io.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import java.net.URL;

import org.olga.rebus.gui.creation.CreateRebus;
import org.olga.rebus.io.*;
import org.olga.rebus.structure.*;
import org.olga.rebus.gui.patterns.*;

/**
 * -.   JMenu.   
 *   .
 * 
 * @author Olga
 * @see JMenu
 *
 */
class RebusMenu extends JMenu {
	private static final long serialVersionUID = 1L;
	// 
	private JFileChooser fc;
	
	//   
	private RebusWriter myRebusWriter;
	//   
	private RebusReader myRebusReader;
	
	// 
	private JMenuBar myMenu = new JMenuBar();
	//,    
	private RebusForm myFrame;
	
	// 
	private JToolBar myTool = new JToolBar();
	
	/**
	 *  .
	 * 
	 * @param frame ,    .
	 */
	RebusMenu(RebusForm frame) {
		myFrame = frame;
		
		//  
		myTool.setFloatable(false);
		myTool.setBorderPainted(false);
		
		//  
		fc = new JFileChooser();
		fc.addChoosableFileFilter(new FileFilter() {
			public boolean accept(File file) {
				if (file.isDirectory()) {
					return true;
				}
				return (file.getName().endsWith("rbs"));
			}
			public String getDescription() {
				return "Files of rebuses (*.rbs)";
			}
		});
		File dirFile = new File(System.getProperty("user.home") + "/rebuses");
		dirFile.mkdir();
		fc.setCurrentDirectory(dirFile);
		
		myRebusWriter = new RebusWriter();
		myRebusReader = new RebusReader();
		
		//    
		myMenu.add(createFileMenu());
		myMenu.add(createEditMenu());
		myMenu.add(createPatternsMenu());
		myMenu.add(createHelpMenu());
		myMenu.add(createLookAndFeelMenu());
		
	}
	
	/**
	 * ,   .
	 * 
	 * @return .
	 */
	JMenuBar getMenu() {
		return myMenu;
	}
	
	/**
	 * ,   .
	 * 
	 * @return  .
	 */
	JToolBar getTool() {		
		return myTool;
	}
	
	private ImageIcon createIcon(String name) {
		URL url = RebusMenu.class.getResource("/org/olga/rebus/gui/images/" + name + ".gif");
		return new ImageIcon(url);
	}
	/*
	 * ,    "File"  
	 *    .
	 */
	private JMenu createFileMenu() {
		JMenuItem exit = new JMenuItem(new ExitAction());
		JMenuItem open = new JMenuItem("Open...");
		JMenuItem save = new JMenuItem("Save...");
		JMenu nnew = new JMenu("New");
		JMenuItem addition = new JMenuItem("Addition...");
		JMenuItem create = new JMenuItem("Other...");
		
		addition.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				new AdditionPattern(myFrame);
			}
		});
		
		create.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				new CreateRebus(myFrame);
			}
		});
		
		JButton bCreate = new JButton(createIcon("new"));
		bCreate.setModel(create.getModel());
		bCreate.setToolTipText("New rebus");
		myTool.add(bCreate);
		
		open.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				fc.setDialogTitle(" ");
				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
				int res = fc.showOpenDialog(RebusMenu.this);
				if (res == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if (!f.getName().endsWith("rbs")) {
						JOptionPane.showMessageDialog(myFrame, "  ,  .", "", 0);
					} else {
						IRebus r = myRebusReader.read(f);
						if (r != null) {
							myFrame.setRebus(r);
						} else {
							JOptionPane.showMessageDialog(myFrame, "   .", "", 0);
						}
					}
				}
			}
		});
		
		JButton bOpen = new JButton(createIcon("open"));
		bOpen.setModel(open.getModel());
		bOpen.setToolTipText("Open rebus");
		myTool.add(bOpen);
		
		save.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (myFrame.getRebus() == null) {
					JOptionPane.showMessageDialog(myFrame, "       .", "", 0);
					return;
				}
				fc.setDialogTitle(" ");
				fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int res = fc.showSaveDialog(RebusMenu.this);
				if (res == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					if (!f.getName().endsWith("rbs")) {
						f = new File(f.getAbsolutePath() + ".rbs");
					} 
					if(f.exists()) {
						f.delete();
					} 
					myRebusWriter.write(myFrame.getRebus(), f);
				}
			}
		});
		
		JButton bSave = new JButton(createIcon("save"));
		bSave.setModel(save.getModel());
		bSave.setToolTipText("Save rebus");
		myTool.add(bSave);
		myTool.addSeparator();
		
		nnew.add(addition);
		nnew.addSeparator();
		nnew.add(create);
				
		JMenu file = new JMenu("File");
		
		file.add(nnew);
		file.add(open);
		file.add(save);
		file.addSeparator();
		file.add(exit);
		return file;
	}
	
	/*
	 * ,    "Edit"  
	 *    .
	 */
	private JMenu createEditMenu() {
		JMenuItem fontColor = new JMenuItem("Font color...");
		JMenuItem backgroundColor = new JMenuItem("Background...");
		JMenu orientation = new JMenu("Orientation");
		JMenuItem horizontal = new JMenuItem("Horizontal");
		JMenuItem vertical = new JMenuItem("Vertical");
		
		fontColor.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				Color c = JColorChooser.showDialog(
						myFrame, "Choose color", Color.BLUE);
				if (c != null) {
					myFrame.setRebusColor(c);
				}
			}
		});
		
		JButton bFontColor = new JButton(createIcon("fontColor"));
		bFontColor.setModel(fontColor.getModel());
		bFontColor.setToolTipText("Font's color");
		myTool.add(bFontColor);
		
		backgroundColor.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				Color c = JColorChooser.showDialog(
						myFrame, "Choose color", Color.LIGHT_GRAY);
				if (c != null) {
					myFrame.setBackgroundColor(c);
				}
			}
		});
		
		JButton bBackground = new JButton(createIcon("background"));
		bBackground.setModel(backgroundColor.getModel());
		bBackground.setToolTipText("Background's color");
		myTool.add(bBackground);
		myTool.addSeparator();
		
		horizontal.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				myFrame.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
			}
		});
		
		JButton bHorizontal = new JButton(createIcon("horizontal"));
		bHorizontal.setModel(horizontal.getModel());
		bHorizontal.setToolTipText("Horizontal orientation");
		myTool.add(bHorizontal);
		
		vertical.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				myFrame.setOrientation(JSplitPane.VERTICAL_SPLIT);
			}
		});
		
		JButton bVertical = new JButton(createIcon("vertical"));
		bVertical.setModel(vertical.getModel());
		bVertical.setToolTipText("Vertical orientation");
		myTool.add(bVertical);
		myTool.addSeparator();
		
		orientation.add(horizontal);
		orientation.add(vertical);
		
		JMenu edit = new JMenu("Edit");
		edit.add(orientation);
		edit.addSeparator();
		edit.add(fontColor);
		edit.add(backgroundColor);
		
		return edit;
	}
	
	/*
	 * ,    "Samples".
	 */
	private JMenu createPatternsMenu() {
		//JMenuItem addition = new JMenuItem("Addition...");
				
		JMenuItem rebus1 = new JMenuItem("Rebus 1");
		JMenuItem rebus2 = new JMenuItem("Rebus 2");
		JMenuItem rebus3 = new JMenuItem("Rebus 3");
		JMenuItem rebus4 = new JMenuItem("Rebus 4");
		JMenuItem rebus5 = new JMenuItem("Rebus 5");
		JMenuItem rebus6 = new JMenuItem("Rebus 6");
		
		rebus1.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_1);
						}
				});	
			}
		});
		
		rebus2.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_2);
						}
				});	
			}
		});
		
		rebus3.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_3);
						}
				});	
			}
		});
		
		rebus4.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_4);
						}
				});	
			}
		});
		
		rebus5.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_7);
						}
				});	
			}
		});
		
		rebus6.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myFrame.setRebus(TestRebuses.REBUS_8);
						}
				});	
			}
		});
		
		JMenu samples = new JMenu("Samples");
		
		samples.add(rebus1);
		samples.add(rebus2);
		samples.add(rebus3);
		samples.add(rebus4);
		samples.add(rebus5);
		samples.add(rebus6);
		
		return samples;
	}
	/*
	 * ,    "Help"   
	 *  .
	 */
	private JMenu createHelpMenu() {
		JMenuItem about = new JMenuItem("About...");
		about.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(myFrame,
						"<html><center><h3> 1.0</h3><p>     .</p><p>   - ,   -  .</p></center></html>", 
						"About", 3);
			}
		});
		
		JButton bHelp = new JButton(createIcon("about"));
		bHelp.setModel(about.getModel());
		bHelp.setToolTipText("About");
		myTool.add(bHelp);
		
		JMenu help = new JMenu("Help");
		help.add(about);
		return help;
	}
	
	/*
	 * ,    "View".
	 */
	private JMenu createLookAndFeelMenu() {
		JMenu menu = new JMenu("View");
        
        ButtonGroup group = new ButtonGroup();
        for (final UIManager.LookAndFeelInfo lafi : UIManager.getInstalledLookAndFeels()) {
            JRadioButtonMenuItem item = new JRadioButtonMenuItem(                
                new AbstractAction(lafi.getName()){
                    private static final long serialVersionUID = 1L;

					public void actionPerformed(ActionEvent event) {
                        try {
                            UIManager.setLookAndFeel(lafi.getClassName());
                            SwingUtilities.updateComponentTreeUI(myFrame);
                        } catch (Exception e) {
                            System.err.println("Cannot set look and feel: " + e.getMessage());
                        }
                    }
                }
            );
            group.add(item);
            menu.add(item);
        }
        return menu;
	}

	/**
	 *  .
	 * 
	 * @author Olga
	 *
	 */
	private class ExitAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		public ExitAction() {
			putValue(NAME, "Exit");
		}
		public void actionPerformed(ActionEvent e) {
			myFrame.writeToFile();
			System.exit(0);
		}
	}
}