package org.olga.rebus.gui.creation;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.olga.rebus.gui.creation.GraphPaper.Cell;
import org.olga.rebus.gui.creation.ModelGraphPaper.Position;
import org.olga.rebus.structure.*;
import org.olga.rebus.structure.Rebus.*;

/**
 * -     .
 * 
 * @author Olga
 * @see IRebus
 *
 */
public class CreateWords extends JPanel {
	private static final long serialVersionUID = 1L;
	// 
	private CreateWordsRepaint myRepaint = new CreateWordsRepaint();
	//  
	private ModelGraphPaper myModel;
	
	// 
	private int myWidth;
	// 
	private int myHeight;
	
	// ,   
	private CreateWordsListener myListener = new CreateWordsListener();
	
	//-   
	private HashMap<Position, Cell> myCells = new HashMap<Position, Cell>();
	// 
	private LinkedList<VisibleLine> myLines = new LinkedList<VisibleLine>();
	// 
	private LinkedList<VisibleSign> mySigns = new LinkedList<VisibleSign>();
	// 
	private LinkedList<VisibleWord> myWords = new LinkedList<VisibleWord>();
	//  
	private LinkedList<VisibleWord> myUnVisibleWords = new LinkedList<VisibleWord>();
	// 
	LinkedList<AbstractRelation> myRelations = new LinkedList<AbstractRelation>();
	
	// 
	private SelectedWord mySelectedWord = new SelectedWord();
	
	//   
	RelationsPanel myRelationsPanel = new RelationsPanel();
	// 
	Paper myPaper;
	
	//-   -
	private HashMap<String, LetterSymbol> myLetters = new HashMap<String, LetterSymbol>();
	//-   -
	private HashMap<Position, AsteriskSymbol> myAsteriskHash = new HashMap<Position, AsteriskSymbol>();

	/**
	 *  - .
	 * 
	 * @author Olga
	 *
	 */
	private class SelectedWord {
		//  
		private boolean isSelect;
		//  
		private boolean isPressed;
		// x  
		private int mySelectX1;
		// y
		private int mySelectY;
		// x  
		private int mySelectX2;
		// x  
		private int myMoveX;
		// y  
		private int myMoveY;
	}
	
	/**
	 * -    .
	 * 
	 * @author Olga
	 *
	 */
	private class Paper extends JPanel {
		private static final long serialVersionUID = 1L;

		Paper(int x1, int y1, int x2, int y2, HashMap<Position, Cell> cells, 
				LinkedList<VisibleLine> lines, LinkedList<VisibleSign> signs) {
			myWidth = x2 - x1 + 1;
			myHeight = y2 - y1 + 1;
			myModel = new ModelGraphPaper(myWidth, myHeight);
			this.setFocusable(false);
			
			Iterator<Position> it = cells.keySet().iterator();
			while(it.hasNext()) {
				Position p = it.next();
				int x = p.myX - x1;
				int y = p.myY - y1;
				if ((x < myWidth) && (y < myHeight)) {
					Position newP = myModel.getCellPosition(x, y);
					Cell cell = cells.get(p);
					Cell cell1 = new Cell(x, y);
					cell1.setSymbol(cell.mySymbol);
					//cell1.setBounds(x * GraphPaper.CELL_SIZE + 1, y * GraphPaper.CELL_SIZE + 1, GraphPaper.CELL_SIZE - 2, GraphPaper.CELL_SIZE - 2);
					myCells.put(newP, cell1);
					//this.add(cell1);
				}
			}
			for (VisibleLine line: lines) {
				int lineX1 = line.getX1() - x1;
				int lineY1 = line.getY1() - y1;
				int lineX2 = line.getX2() - x1;
				int lineY2 = line.getY2() - y1;
				myLines.add(Rebus.createVisibleLine(lineX1 * 2 - 1, lineY1 * 2 - 1, lineX2 * 2 - 1, lineY2 * 2 - 1));
			}
			for (VisibleSign sign: signs) {
				int signX = sign.getX() - x1 * 2;
				int signY = sign.getY() - y1 * 2;
				mySigns.add(Rebus.createVisibleSign(sign.getElement(), signX - 1, signY - 1));
			}
			
			this.addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
				public void mousePressed(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
				public void mouseReleased(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
				public void mouseEntered(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
				public void mouseExited(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
			});
			
			this.addMouseMotionListener(new MouseMotionListener() {
				public void mouseDragged(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
				public void mouseMoved(MouseEvent e) {
					e.setSource(CreateWords.this);
					CreateWords.this.dispatchEvent(e);
				}
			});
			
			this.setBackground(new Color(255, 255, 200));
			this.setLayout(null);
			
			this.setAlignmentX(Component.CENTER_ALIGNMENT);
			this.setAlignmentY(Component.CENTER_ALIGNMENT);
			setPreferredSize(new Dimension(GraphPaper.CELL_SIZE * myWidth, GraphPaper.CELL_SIZE * myHeight));
			setVisible(true);
		}
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Font font = g.getFont();
			g.setFont(new Font(font.getName(), Font.BOLD, font.getStyle()));
			g.setColor(new Color(244, 223, 195));
			if (mySelectedWord.isSelect) {
				int x1 = mySelectedWord.mySelectX1 * GraphPaper.CELL_SIZE;
				int y1 = mySelectedWord.mySelectY * GraphPaper.CELL_SIZE;
				int x2 = mySelectedWord.mySelectX2 * GraphPaper.CELL_SIZE;
				g.fillRect(x1 + 1, y1 + 1, x2 - x1 + GraphPaper.CELL_SIZE, GraphPaper.CELL_SIZE);
			}
			g.setColor(new Color(242, 205, 158));
			for (int i = 1; i < myHeight; i++) {
				g.drawLine(0, i * GraphPaper.CELL_SIZE, GraphPaper.CELL_SIZE * myWidth, i * GraphPaper.CELL_SIZE);
			}
			for (int i = 1; i < myWidth; i++) {
				g.drawLine(i * GraphPaper.CELL_SIZE, 0, i * GraphPaper.CELL_SIZE, GraphPaper.CELL_SIZE * myHeight);
			}
			g.setColor(new Color(Color.BLUE.getRGB()));
			for (VisibleLine line: myLines) {
				int x1 = (line.getX1() + 1) * GraphPaper.CELL_SIZE / 2;
				int x2 = (line.getX2() + 1) * GraphPaper.CELL_SIZE / 2;
				int y1 = (line.getY1() + 1) * GraphPaper.CELL_SIZE / 2;
				int y2 = (line.getY2() + 1) * GraphPaper.CELL_SIZE / 2;
				g.drawLine(x1, y1, x2, y2);
			}
			g.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
			for (VisibleSign sign: mySigns) {
				int xx = (sign.getX() + 1) * GraphPaper.CELL_SIZE / 2;
				int yy = (sign.getY() + 1) * GraphPaper.CELL_SIZE / 2;
				if (sign.getElement() == '*') {
					g.drawString("" + sign.getElement(), xx - 4, yy + 10);
				} else {
					g.drawString("" + sign.getElement(), xx - 4, yy + 6);
				}
			}
			g.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
			paintWord(g, 0, 0);
			
			g.setFont(new Font(font.getName(), font.getStyle(), font.getSize()));
			for (Position p: myCells.keySet()) {
				Cell cell = myCells.get(p);
				g.drawString(cell.mySymbol.toString(), cell.myX * GraphPaper.CELL_SIZE + 8, cell.myY * GraphPaper.CELL_SIZE + 15);
			}
		}
	}
			
	/**
	 *  .
	 * 
	 * @param x1  x     .
	 * @param y1  y     .
	 * @param x2  x     .
	 * @param y2  y     .
	 * @param cells -  .
	 * @param lines  .
	 * @param signs  .
	 */
	public CreateWords(int x1, int y1, int x2, int y2, HashMap<Position, Cell> cells, 
			LinkedList<VisibleLine> lines, LinkedList<VisibleSign> signs) {
		
		myPaper = new Paper(x1, y1, x2, y2, cells, lines, signs);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(myPaper);
		this.add(myRelationsPanel);
		
		this.addMouseListener(myListener);
		this.addMouseMotionListener(myListener);
		
		this.setBackground(new Color(255, 255, 200));
		this.setVisible(true);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		int x = - myPaper.getX();
		int y = - myPaper.getY();
		paintWord(g, x, y);
	}
	
	/*
	 *  -     .
	 */
	private class PaintPanel extends JPanel {
		private static final long serialVersionUID = 1L;
		
		PaintPanel() {
			this.setBackground(GraphPaper.BACKGROUND_COLOR);
		}
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			int x = this.getX();
			int y = myRelationsPanel.getY() + this.getY();
			paintWord(g, x, y);
		}
	}
	
	/*
	 * ,     
	 *    .
	 */
	private void paintWord(Graphics g, int x, int y) {
		if (mySelectedWord.isPressed) {
			boolean isFirst = true;
			StringBuffer sb = new StringBuffer();
			for (int i = mySelectedWord.mySelectX1; i <= mySelectedWord.mySelectX2; i++) {
				Position p = myModel.getCellPosition(i, mySelectedWord.mySelectY);
				Cell cell = myCells.get(p);
				if (cell != null) {
					sb.append(cell.mySymbol);
					isFirst = false;
				} else if (!isFirst) {
					sb.append("0");
				}
			}
			Font font = g.getFont();
			g.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
			g.setColor(Color.BLUE);
			x += sb.toString().length() * 5;
			g.drawString(sb.toString(), mySelectedWord.myMoveX - x, mySelectedWord.myMoveY - y);
		}
	}
	
	/**
	 * -   .
	 * 
	 * @author Olga
	 *
	 */
	private class CreateWordsListener implements MouseListener, MouseMotionListener {
		
		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			mySelectedWord.isSelect = false;
			mySelectedWord.isPressed = false;
			SwingUtilities.invokeLater(myRepaint);
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			int x = e.getX();
			int y = e.getY();
			int x1 = mySelectedWord.mySelectX1 * GraphPaper.CELL_SIZE;
			int y1 = mySelectedWord.mySelectY * GraphPaper.CELL_SIZE;
			int x2 = (mySelectedWord.mySelectX2 + 1) * GraphPaper.CELL_SIZE;
			int y2 = y1 + GraphPaper.CELL_SIZE;
			
			if ((x1 < x) && (x2 > x) && (y1 < y) && (y2 > y) && mySelectedWord.isSelect) {
				mySelectedWord.isPressed = true;
				return;
			} 
			
			mySelectedWord.mySelectX1 = e.getX() / GraphPaper.CELL_SIZE;
			mySelectedWord.mySelectY = e.getY() / GraphPaper.CELL_SIZE;
			if (mySelectedWord.mySelectX1 >= myWidth) {
				mySelectedWord.mySelectX1 = myWidth - 1;
			}
			if (mySelectedWord.mySelectY >= myHeight) {
				mySelectedWord.mySelectY = myHeight - 1;
			}		
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent e) {	
			Component component = CreateWords.this.getComponentAt(e.getX(), e.getY());
			if (component instanceof RelationsPanel) {
				component = component.getComponentAt(e.getX(), e.getY() - myPaper.getHeight());
				if (component instanceof OperandsPanel) {
					component = component.getComponentAt(e.getX(), e.getY() - myPaper.getHeight());
					if (component instanceof JPanel) {
						component = component.getComponentAt(e.getX(), e.getY() - myPaper.getHeight() - myRelationsPanel.myOperandsPanel.myCountPanel.getHeight());
						if (component instanceof Operand) {
							if (mySelectedWord.isPressed) {
								((Operand) component).myWord = Rebus.createVisibleWord(createWord(), mySelectedWord.mySelectX1 * 2, mySelectedWord.mySelectY * 2);
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										myRelationsPanel.repaint();
									}
								});
							}
						}
					}
				}	
			}
				
			mySelectedWord.isPressed = false;			
			SwingUtilities.invokeLater(myRepaint);	
		}

		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mouseMoved(MouseEvent e) {}
		
		/* (non-Javadoc)
		 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
		 */
		public void mouseDragged(MouseEvent e) {
			if (myCells.get(myModel.getCellPosition(mySelectedWord.mySelectX1, mySelectedWord.mySelectY)) == null) {
				return;
			}
			if (mySelectedWord.isPressed) {
				mySelectedWord.myMoveX = e.getX();
				mySelectedWord.myMoveY = e.getY();
				SwingUtilities.invokeLater(myRepaint);
				return;
			}
			mySelectedWord.mySelectX2 = e.getX() / GraphPaper.CELL_SIZE;
			if (mySelectedWord.mySelectX2 >= myWidth) {
				mySelectedWord.mySelectX2 = myWidth - 1;
			}
			mySelectedWord.isSelect = true;
			SwingUtilities.invokeLater(myRepaint);	
		}		
	}
	
	/*
	 * ,        .
	 */
	private AbstractWord createWord() {
		boolean unVisible = false;
		int len = mySelectedWord.mySelectX2 - mySelectedWord.mySelectX1 + 1;
		for (VisibleWord word: myWords) {
			int x = word.getX() / 2;
			int y = word.getY() / 2;
			int length = word.getElement().toString().length();
			if ((mySelectedWord.mySelectY == y) && (length == len) && (mySelectedWord.mySelectX1 == x)) {
				return word.getElement();
			}
		}
		AbstractSymbol[] word = new AbstractSymbol[len];
		int k = 0;
		for (int i = mySelectedWord.mySelectX1; i <= mySelectedWord.mySelectX2; i++) {
			Position p = myModel.getCellPosition(i, mySelectedWord.mySelectY);
			if (!myCells.containsKey(p)) {
				word[k++] = Rebus.createNumberSymbol(0);
				unVisible = true;
				continue;
			}
			String symb = myCells.get(p).mySymbol.toString();
			try {
				int n = Integer.parseInt(symb);
				word[k++] = Rebus.createNumberSymbol(n);
			} catch (NumberFormatException e1) {
				if (symb.equals("*")) {
					if (!myAsteriskHash.containsKey(p)) {
						AsteriskSymbol s = Rebus.createAsteriskSymbol();
						word[k++] = s;
						myAsteriskHash.put(p, s);
					} else {
						word[k++] = myAsteriskHash.get(p);
					}
				} else if (symb.equals("")) {
				} else {
					if (!myLetters.containsKey(symb)) {
						LetterSymbol s = Rebus.createLetterSymbol(symb);
						word[k++] = s;
						myLetters.put(symb, s);
					} else {
						word[k++] = myLetters.get(symb);
					}
				}
			}
		}
		AbstractSymbol[] word1 = new AbstractSymbol[k];
		for (int i = 0; i < k; i++) {
			word1[i] = word[i];
		}
		AbstractWord newWord = Rebus.createWord(word1);
		VisibleWord w = Rebus.createVisibleWord(newWord, mySelectedWord.mySelectX1 * 2, mySelectedWord.mySelectY * 2);
		myWords.add(w);
		if(unVisible) {
			myUnVisibleWords.add(w);
		}
		return newWord;
	}
	
	/**
	 * ,   .
	 * 
	 * @return  .
	 */
	public IRebus getRebus() {
		IRebus rebus = new Rebus(myRelations, myWords, mySigns, myLines);
		LinkedList<Subword> subwords = getSubwords();
		if (subwords != null) {
			rebus.setSubwords(subwords);
		}
		for (VisibleWord w: myUnVisibleWords) {
			myWords.remove(w);
		}
		return rebus;
	}
	
	/*
	 * ,      
	 *    .
	 */
	private LinkedList<Subword> getSubwords() {
		int len = myWords.size();
		LinkedList<Subword> subwords = new LinkedList<Subword>();
		for (int i = 0; i < len; i++) {
			VisibleWord curWord = myWords.removeFirst();
			int x = curWord.getX();
			int y = curWord.getY();
			int length = curWord.getElement().toString().length();
			boolean f = true;
			for (VisibleWord word: myWords) {
				int x1 = word.getX();
				int y1 = word.getY();
				int length1 = word.getElement().toString().length();
				if ((y == y1) && (length < length1) && (x >= x1) && (x + length *  2<= x1 + length1 * 2)) {
					subwords.add(Rebus.createSubword(word.getElement(), curWord.getElement(), (x - x1) / 2, (x - x1) / 2 + length));
					if (!myUnVisibleWords.contains(word)) {
						f = false;
					}
					break;
				} 
			}
			if (f) {
				myWords.add(curWord);
			}
		}
		return subwords;
	}
	
	/*
	 *     .
	 */
	private class RelationsPanel extends JPanel {
		private static final long serialVersionUID = 1L;
				
		private JButton myCreateButton;
		private JButton myDeleteButton;
		
		LinkedList<Position> myPositions = new LinkedList<Position>(); 
		
		private RelationsList myList = new RelationsList(new DefaultListModel());
		
		private OperandsPanel myOperandsPanel = new OperandsPanel();
		
		public RelationsPanel() {
			
			myCreateButton = new JButton("Create");
			myCreateButton.addActionListener(new AbstractAction() {
				private static final long serialVersionUID = 1L;
				public void actionPerformed(ActionEvent e) {
					for (Operand op: myOperandsPanel.myOperands) {
						if (op.myWord == null) {
							JOptionPane.showMessageDialog(CreateWords.this, "   .", "", 0);
							return;
						}
					}
					int start = myRelations.size();
					Operand wordOp1 = myOperandsPanel.myOperands.get(0);
					Operand wordOp2 = myOperandsPanel.myOperands.get(1);
					Operand wordRes;
					AbstractWord op1 = wordOp1.myWord.getElement();
					AbstractWord op2 = wordOp2.myWord.getElement();
					AbstractWord res;
					String sign = (String) myOperandsPanel.myCombos.get(0).getModel().getSelectedItem();
					String relation = "";
					
					if (myOperandsPanel.myOperands.size() == 2) {
						myRelations.add(Rebus.createEqualityRelation(op1, op2));
						relation = op1 + " = " + op2;
					} else {
						StringBuffer sb = new StringBuffer();
						
						int count = myOperandsPanel.myOperands.size();
						DummyWord[] dummies = new DummyWord[count - 3];
						for (int i = 0; i < dummies.length; i++) {
							dummies[i] = Rebus.createDummyWord();
						}
						if (dummies.length != 0) {
							res = dummies[0];
						} else {
							wordRes = myOperandsPanel.myOperands.get(2);
							res = wordRes.myWord.getElement();
						}
						sign = (String) myOperandsPanel.myCombos.get(0).getModel().getSelectedItem();
						if (sign.equals("+")) {
							myRelations.add(Rebus.createAdditionRelation(op1, op2, res));
							sb.append(op1).append(" + ").append(op2);
						} else if (sign.equals("-")) {
							myRelations.add(Rebus.createAdditionRelation(op2, res, op1));
							sb.append(op1).append(" - ").append(op2);
						} else if (sign.equals("*")) {
							myRelations.add(Rebus.createMultiplyRelation(op1, op2, res));
							sb.append(op1).append(" * ").append(op2);
						} else if (sign.equals("/")) {
							myRelations.add(Rebus.createMultiplyRelation(op2, res, op1));
							sb.append(op1).append(" / ").append(op2);
						}
						for (int i = 2; i < count - 1; i++) {
							op1 = dummies[i - 2]; 
							sign = (String) myOperandsPanel.myCombos.get(i - 1).getModel().getSelectedItem();	
							op2 = myOperandsPanel.myOperands.get(i).myWord.getElement();
							if (dummies.length > (i - 1)) {
								res = dummies[i - 1];
							} else {
								wordRes = myOperandsPanel.myOperands.get(i + 1);
								res = wordRes.myWord.getElement();
							}
							
							if (sign.equals("+")) {
								myRelations.add(Rebus.createAdditionRelation(op1, op2, res));
								sb.append(" + ").append(op2);
							} else if (sign.equals("-")) {
								myRelations.add(Rebus.createAdditionRelation(op2, res, op1));
								sb.append(" - ").append(op2);
							} else if (sign.equals("*")) {
								myRelations.add(Rebus.createMultiplyRelation(op1, op2, res));
								sb.append(" * ").append(op2);
							} else if (sign.equals("/")) {
								myRelations.add(Rebus.createMultiplyRelation(op2, res, op1));
								sb.append(" / ").append(op2);
							}
							
						}
						sb.append(" = ").append(myOperandsPanel.myOperands.getLast().myWord.getElement());
						relation = sb.toString();
					}
					
					int len = Integer.parseInt(((SpinnerNumberModel)myOperandsPanel.myCount.getModel()).getNumber().toString());
					myPositions.add(new Position(start, len - 2));
					myList.addListElement(relation);
					for (Operand op: myOperandsPanel.myOperands) {
						op.myWord = null;
					}
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							myList.repaint();
							myOperandsPanel.repaint();
						}
					});
				}
			});
				
			myDeleteButton = new JButton("Delete");
			myDeleteButton.addActionListener(new AbstractAction() {
				private static final long serialVersionUID = 1L;
				public void actionPerformed(ActionEvent e) {
					int i = myList.getSelectedIndex();
					if ((i >= 0) && (i < myList.getListSize())) {
						
						
						myList.removeElement(i);
						Position p = myPositions.remove(i);
						for (int j = 0; j < p.myY; j++) {
							myRelations.remove(p.myX);
						}
						int l = myPositions.size();
						for (int k = p.myX; k < l; k++) {
							myPositions.get(k).myX -= p.myY; 
						}
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								myList.repaint();
							}
						});
					}
				}
			});
			
			PaintPanel buttonPanel = new PaintPanel();
			buttonPanel.setBackground(GraphPaper.BACKGROUND_COLOR);
			buttonPanel.add(myCreateButton);
			buttonPanel.add(myDeleteButton);
			buttonPanel.setMaximumSize(new Dimension(180, 50));
			
			PaintPanel listPanel = new PaintPanel();
			JScrollPane scroll = new JScrollPane(myList);
			scroll.setPreferredSize(new Dimension(300, 92));
			scroll.setMaximumSize(new Dimension(300, 92));
			scroll.setMinimumSize(new Dimension(300, 92));
			listPanel.add(scroll);
			listPanel.setBackground(GraphPaper.BACKGROUND_COLOR);
			
			
			this.setLayout(new BoxLayout(CreateWords.RelationsPanel.this, BoxLayout.Y_AXIS));
			this.add(myOperandsPanel);
			this.add(buttonPanel);
			this.add(scroll);
			this.setBackground(GraphPaper.BACKGROUND_COLOR);
			this.setVisible(true);
		}
		
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			int x = myPaper.getX();
			int y = myPaper.getHeight();
			paintWord(g, x, y);
		}
		
		private class RelationsList extends JList {
			private static final long serialVersionUID = 1L;

			private DefaultListModel myListModel;
			
			RelationsList(DefaultListModel m) {
				super(m);
				myListModel = m;
				this.setBorder(new LineBorder(new Color(149, 49, 18)));
				this.setBackground(new Color(244, 223, 195));
				this.setForeground(Color.BLUE);
				Font f = this.getFont();
				this.setFont(new Font(f.getName(), Font.TRUETYPE_FONT, f.getSize()));
				
			}
			
			int getListSize() {
				return myListModel.getSize();
			}
			void removeElement(int index) {
				myListModel.remove(index);
			}
			void addListElement(String element) {
				myListModel.addElement(element);
			}			
		}				
	}
	
	/*
	 *   .
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return new Dimension(700, getPreferredSize().height);
	}

	/*
	 *   .
	 */
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		return 20;
	}

	/*
	 *   .
	 */
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		return 100;
	}

	/*
	 *   .
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/*
	 *   .
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	
	/*
	 *       .
	 */
	private class OperandsPanel extends PaintPanel {
		private static final long serialVersionUID = 1L;
		
		private LinkedList<Operand> myOperands = new LinkedList<Operand>();
		private String[] mySigns = new String[] {"+", "-", "*", "/"};
		private String[] myEqual = new String[] {"="};
		private JSpinner myCount;
		private LinkedList<JComboBox> myCombos = new LinkedList<JComboBox>();
		private String[] myCountValues = new String[8];
		private PaintPanel myPanel = new PaintPanel();
		private PaintPanel myCountPanel = new PaintPanel();
		
		public OperandsPanel() {
			super();
			for (int i = 0; i < myCountValues.length; i++) {
				myCountValues[i] = "" + (i + 2);
			}
			final SpinnerNumberModel model = new SpinnerNumberModel(3, 2, 9, 1);
			
			myCount = new JSpinner(model);
			JFormattedTextField tf =
		        ((JSpinner.DefaultEditor)myCount.getEditor()).getTextField();
		    tf.setEditable(false);
		    Font font = tf.getFont();
		    tf.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
		    tf.setBackground(GraphPaper.BACKGROUND_COLOR);
		    tf.setForeground(new Color(149, 49, 18));
			myCount.addChangeListener(new ChangeListener() {

				public void stateChanged(ChangeEvent e) {
					String s = model.getNumber().toString();
					
					int count = Integer.parseInt(s);
					change(count);
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							CreateWords.OperandsPanel.this.repaint();
							CreateWords.this.repaint();
						}
					});
				}

			
			});
			this.setLayout(new BoxLayout(CreateWords.OperandsPanel.this, BoxLayout.Y_AXIS));
			JLabel label = new JLabel("Number of operands: ");
			label.setForeground(new Color(149, 49, 18));
			Font f = label.getFont();
			label.setFont(new Font(f.getName(), Font.BOLD, f.getSize() + 3));
			
			myCountPanel.add(label);
			myCountPanel.add(myCount);
			//myCountPanel.setBackground(GraphPaper.BACKGROUND_COLOR);
			myCountPanel.setMaximumSize(new Dimension(350, 50));
			//myPanel.setBackground(GraphPaper.BACKGROUND_COLOR);
			this.add(myCountPanel);
			
			this.add(myPanel);
			change(2);
			change(3);
			
			this.setBackground(GraphPaper.BACKGROUND_COLOR);
			this.setVisible(true);
		}
		
		private void change(int count) {
			int size = myOperands.size();
			Dimension operandDimension;
			if (count <= 5) {
				operandDimension = new Dimension((650 - 50 * (count - 1)) / count, 50);
				if (count == 5) {
					myPanel.setPreferredSize(new Dimension(myRelationsPanel.getWidth(), 60));
				}
			} else {
				operandDimension = new Dimension(90, 50);
				if (count == 6) {
					myPanel.setPreferredSize(new Dimension(myRelationsPanel.getWidth(), 120));
				}
			}
			for (JPanel panel: myOperands) {
				panel.setPreferredSize(operandDimension);
			}
			if (size < count) {
				for (int i = size + 1; i <= count; i++) {
					if ((count != 2)) {
						JComboBox combo = new JComboBox(mySigns);
						combo.setPrototypeDisplayValue("00");
						myCombos.add(myCombos.size() - 1, combo);
						JComboBox c = myCombos.getLast();
						Operand o = myOperands.getLast();
						myPanel.remove(c); 
						myPanel.remove(o);
						myPanel.add(combo);
						myPanel.add(o);
						myPanel.add(c);
					} 
					Operand panel = new Operand(i);
					panel.setPreferredSize(operandDimension);
					
					myOperands.add(panel);
					myPanel.add(panel);
					if ((count == 2) && (i == 1)) {
						JComboBox combo = new JComboBox(myEqual);
						combo.setPrototypeDisplayValue("00");
						myCombos.add(combo);
						myPanel.add(combo);
					}
				}
			} else if (size > count) {
				for (int i = 0; i < size - count; i++) {
					Operand panel = myOperands.removeLast();
					Operand panel1 = myOperands.removeLast();
					JComboBox combo = myCombos.removeLast();
					JComboBox combo1 = myCombos.removeLast();
					
					myPanel.remove(panel);
					myPanel.remove(combo);
					myPanel.remove(panel1);
					myPanel.remove(combo1);
					myPanel.add(combo);
					myPanel.add(panel1);
					myOperands.add(panel1);
					myCombos.add(combo);
				}
			}
			myPanel.revalidate();
		}
	}
	
	/*
	 *  -.
	 */
	private class Operand extends JPanel {
		private static final long serialVersionUID = 1L;
		
		final Color myOperandColor = new Color(149, 49, 18);
		VisibleWord myWord;
		public Operand(int i) {
			this.setBackground(GraphPaper.BACKGROUND_COLOR);
			this.setBorder(new TitledBorder(new LineBorder(myOperandColor),
					"Operand " + i, TitledBorder.LEADING, TitledBorder.TOP, 
					null, myOperandColor));
			this.setVisible(true);
		}
		
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			if (myWord != null) {
				g.setColor(Color.BLUE);
				g.drawString(myWord.getElement().toString(), 10, 30);
			}
			int x = this.getX() - myPaper.getX();
			int y = this.getY() - myPaper.getY() + myRelationsPanel.myOperandsPanel.myCountPanel.getHeight() + myPaper.getHeight();
			
			paintWord(g, x, y);
		}
	}
	
	/**
	 * ,      .
	 * 
	 * @return     .
	 */
	RelationsPanel getRelationsPanel() {
		return myRelationsPanel;
	}
	
	/*
	 *  -.
	 */
	private class CreateWordsRepaint implements Runnable {
		public void run() {
			CreateWords.this.repaint();
		}
	}
}
