package org.olga.rebus.io;

import java.io.*;
import java.util.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.olga.rebus.structure.*;
import org.olga.rebus.structure.Rebus.*;
import org.w3c.dom.*;

/**
 *     .
 * 
 * @author Olga
 * @see IRebus
 *
 */
public class RebusWriter {

	// 
	private IRebus myRebus;	
		
	//,    .
	private Document makeDoc() throws IOException, ParserConfigurationException {
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder parser = factory.newDocumentBuilder();
		Document doc = parser.newDocument();
		Node root = doc.createElement("Rebus");
		doc.appendChild(root);
		
		int id = 0;
		HashMap<AbstractWord, Element> hash = new HashMap<AbstractWord, Element>();
		
		Collection<AbstractRelation> relations = myRebus.getRelations();
		for(AbstractRelation relation: relations) {			
			AbstractWord op1 = relation.getOp1();
			AbstractWord op2 = relation.getOp2();
			AbstractWord result = relation.getResult();
			if (!hash.containsKey(op1)) {
				Element element = doc.createElement("word");
				element.setAttribute("id", String.valueOf(id++));
				element.setAttribute("type", op1.getClass().getSimpleName());
				element.setAttribute("name", op1.toString());
				root.appendChild(element);
				hash.put(op1, element);
			}
			if (!hash.containsKey(op2)) {
				Element element = doc.createElement("word");
				element.setAttribute("id", String.valueOf(id++));
				element.setAttribute("type", op2.getClass().getSimpleName());
				element.setAttribute("name", op2.toString());
				root.appendChild(element);
				hash.put(op2, element);
			}
			if ((result != null) && (!hash.containsKey(result))) {
				Element element = doc.createElement("word");
				element.setAttribute("id", String.valueOf(id++));
				element.setAttribute("type", result.getClass().getSimpleName());
				element.setAttribute("name", result.toString());
				root.appendChild(element);
				hash.put(result, element);
			}
		}
		
		Collection<Subword> substrings = myRebus.getSubwords();
		if (substrings != null) {
			for(Subword sub: substrings) {
				AbstractWord toWord = sub.getToWord();
				AbstractWord fromWord = sub.getFromWord();
				Element toElement = hash.get(toWord);
				Element fromElement = hash.get(fromWord);
				toElement.setAttribute("substring", fromElement.getAttribute("id"));
				toElement.setAttribute("from", String.valueOf(sub.getFrom()));
				toElement.setAttribute("to", String.valueOf(sub.getTo()));
			}
		}
				
		Collection<VisibleWord> words = myRebus.getVisibleWords();
		for(VisibleWord word: words) {
			AbstractWord w = word.getElement();
			Element element;
			if (!hash.containsKey(w)) {
				element = doc.createElement("word");
				element.setAttribute("id", String.valueOf(id++));
				element.setAttribute("type", w.getClass().getSimpleName());
				element.setAttribute("name", w.toString());
				root.appendChild(element);
			} else {
				element = hash.get(w);
			}
			element.setAttribute("show", "true");
			element.setAttribute("positionX", String.valueOf(word.getX()));
			element.setAttribute("positionY", String.valueOf(word.getY()));
		}
		
		Collection<AbstractWord> enums = myRebus.getEnumWords();
		for (AbstractWord enumWord: enums) {
			Element element = hash.get(enumWord);
			element.setAttribute("enum", "true");
		}
		
		for(AbstractRelation relation: relations) {
			Element element = doc.createElement("relation");
			AbstractWord op1 = relation.getOp1();
			AbstractWord op2 = relation.getOp2();
			AbstractWord result = relation.getResult();
			Element elOp1 = hash.get(op1);
			Element elOp2 = hash.get(op2);
			element.setAttribute("operand1", elOp1.getAttribute("id"));
			element.setAttribute("operand2", elOp2.getAttribute("id"));
			if (result != null) {
				Element elResult = hash.get(result);
				element.setAttribute("result", elResult.getAttribute("id"));
			}
			element.setAttribute("id", String.valueOf(id++));
			element.setAttribute("type", relation.getClass().getSimpleName());
			root.appendChild(element);
		}
		
		Collection<VisibleSign> signs = myRebus.getSigns();
		for(VisibleSign sign: signs) {
			Element ndSign = doc.createElement("show");
			ndSign.setAttribute("type", "sign");
			ndSign.setAttribute("id", String.valueOf(id++));
			ndSign.setAttribute("name", String.valueOf(sign.getElement()));
			ndSign.setAttribute("positionX", String.valueOf(sign.getX()));
			ndSign.setAttribute("positionY", String.valueOf(sign.getY()));
			
			root.appendChild(ndSign);
		}
		
		Collection<VisibleLine> lines = myRebus.getLines();
		if (lines != null) {
			for(VisibleLine line: lines) {
				Element ndLine = doc.createElement("show");
				ndLine.setAttribute("type", "line");
				ndLine.setAttribute("id", String.valueOf(id++));
				ndLine.setAttribute("positionX1", String.valueOf(line.getX1()));
				ndLine.setAttribute("positionY1", String.valueOf(line.getY1()));
				ndLine.setAttribute("positionX2", String.valueOf(line.getX2()));
				ndLine.setAttribute("positionY2", String.valueOf(line.getY2()));
			
				root.appendChild(ndLine);
			}
		}
		return doc;
	}
	
	/** 
	 * ,    .
	 * 
	 * @param r ,  .
	 * @param file ,    .
	 */
	public void write(IRebus r, File file) {
		myRebus = r;
		try {
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer transformer = tf.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			
			Document document = this.makeDoc();
						
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(file);
			
			transformer.transform(source, result);
			OutputStream os = result.getOutputStream();
			if (os != null) {
				os.close();
			}
			
		} catch (ParserConfigurationException e) {
			System.out.println(e.getMessage());
		} catch (IOException e1) {
			System.out.println(e1.getMessage());
		} catch (TransformerConfigurationException e2) {
			System.out.println(e2.getMessage());
		} catch (TransformerException e3) {
			System.out.println(e3.getMessage());
		}
	}
}
